#include <WiFi.h>
#include <DHT.h>
#include <PubSubClient.h>
#include "config.h"
#include <string.h>
#include <Wire.h>

#define LDR_GPIO 7
#define DHT11_Pin 10
#define Raindrops_Pin 6
#define TOPIC_RAINDROPS TOPIC_PREFIX "/raindrops"
#define TOPIC_LIGHT    TOPIC_PREFIX "/light"
#define TOPIC_TEMPERATURE TOPIC_PREFIX "/temperature"
#define TOPIC_HUMIDITY TOPIC_PREFIX "/humidity"

WiFiClient wifiClient;
PubSubClient mqtt(MQTT_BROKER, 1883, wifiClient);
uint32_t last_publish;
DHT dht(DHT11_Pin, DHT11);

void connect_wifi() {
  printf("WiFi MAC address is %s\n", WiFi.macAddress().c_str());
  printf("Connecting to WiFi %s.\n", WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    printf(".");
    fflush(stdout);
    delay(500);
  }
  printf("\nWiFi connected.\n");
}

void connect_mqtt() {
  printf("Connecting to MQTT broker at %s.\n", MQTT_BROKER);
  if (!mqtt.connect("", MQTT_USER, MQTT_PASS)) {
    printf("Failed to connect to MQTT broker.\n");
    for (;;) {} // wait here forever
  }
  printf("MQTT broker connected.\n");
}

void setup() {
  connect_wifi();
  connect_mqtt();                                               
  last_publish = 0;
  dht.begin();
  delay(2000);
}

void loop() {
  uint32_t now = millis();
  if (now - last_publish >= 1000) {
    int raindrops_level = analogRead(Raindrops_Pin); //Publish analogRead from raindrops sensor.
    String payload(raindrops_level);
    mqtt.publish(TOPIC_RAINDROPS, payload.c_str());

    int level = (analogRead(LDR_GPIO)); //Publish analogRead from LDR.
    String payload2(level);
    mqtt.publish(TOPIC_LIGHT, payload2.c_str());

    float temperature = dht.readTemperature(); //Publish analogRead DHT11.
    String payload3(temperature);
    mqtt.publish(TOPIC_TEMPERATURE, payload3.c_str());

    float humidity = dht.readHumidity(); //Publish analogRead from DHT11.
    String payload4(humidity);
    mqtt.publish(TOPIC_HUMIDITY, payload4.c_str());
    last_publish = now;
  }
}