#include <WiFi.h>
#include <PubSubClient.h>
#include "config.h"
#include <string.h>
#include <Wire.h>

#define In1 15 //In1 Relay
#define In2 7 //In2 Relay
#define sw1 5 //Collecting switch
#define sw2 6 //Hanging switch
#define TOPIC_CONTROL TOPIC_PREFIX "/control"
#define TOPIC_STATUS TOPIC_PREFIX "/status"
#define TOPIC_AUTOMATION TOPIC_PREFIX "/automation"

WiFiClient wifiClient;
PubSubClient mqtt(MQTT_BROKER, 1883, wifiClient);
int status;


int relay(int value, int status){
  int relays[2] = {In2, In1};
  int switches[2] = {sw1, sw2};

  while (digitalRead(switches[value]) == 0){ //Power relay to collect or hang.
    pinMode(relays[value], OUTPUT);
  }
  pinMode(relays[value], INPUT);
  pinMode(relays[1-value], OUTPUT);
  delay(200);
  pinMode(relays[1-value], INPUT);

  status = 1 - status; //Switch status and publish.
  String payload(status);
  mqtt.publish(TOPIC_STATUS, payload.c_str());
  return status;
}

void connect_wifi() {
  printf("WiFi MAC address is %s\n", WiFi.macAddress().c_str());
  printf("Connecting to WiFi %s.\n", WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    printf(".");
    fflush(stdout);
    delay(500);
  }
  printf("\nWiFi connected.\n");
}

void connect_mqtt() {
  printf("Connecting to MQTT broker at %s.\n", MQTT_BROKER);
  if (!mqtt.connect("", MQTT_USER, MQTT_PASS)) {
    printf("Failed to connect to MQTT broker.\n");
    for (;;) {} // wait here forever
  }
  mqtt.setCallback(mqtt_callback);
  mqtt.subscribe(TOPIC_CONTROL);
  printf("MQTT broker connected.\n");
}

void mqtt_callback(char* topic, byte* payload, unsigned int length) {
  payload[length] = 0; // null-terminate the payload to treat it as a string
  if (strcmp(topic, TOPIC_CONTROL) == 0){
    int value = atoi((char*)payload);
    if (value == 0 and status == 1){ //Collect
      status = relay(value, status);
    }
    else if (value == 1 and status == 0){ //Hang
      status = relay(value, status);
    }
  }
}

void setup() {
  connect_wifi();
  connect_mqtt();
  pinMode(sw1, INPUT);
  pinMode(sw2, INPUT);
  status = relay(0, 1); { // Set initial status as "Collected" and publish.
  String auto_off(0); // Set initial automation as "off".
  mqtt.publish(TOPIC_AUTOMATION ,auto_off.c_str());
  delay(2000);
}

void loop() {
  mqtt.loop();
}